import time

from netforce.model import Model, fields, get_model
from netforce.access import get_active_company, get_active_user, set_active_user

PATIENT_TYPE={
    "mg":"Medical Government",
    "sc":"Social Security",
    "nhso":"NHSO (30฿)",
    "personal": "Personal",
    "others": "Others",
}

class Patient(Model):
    _name="clinic.patient"
    _string="Patient"
    _audit_log=True
    _multi_company=True

    def _get_age(self,ids,context):
        res={}
        year_now=int(time.strftime("%Y"))
        for obj in self.browse(ids):
            age=0
            if obj.birthday:
                year_bd=int(obj.birthday[0:4])
                age=year_now-year_bd
            res[obj.id]=age
        return res # -> {1: 30, 2: 45,.....}

    _fields={
        "type": fields.Selection([("mg","Medical Government"),("sc","Social Security"),("nhso","NHSO (30฿)"),("personal","Personal"),("others","Others")],"Patient Type",required=True),
        "number": fields.Char("Patient No.",required=True,search=True),
        "hn": fields.Char("HN",search=True),
        "name": fields.Char("Name",required=True,search=True),
        "reg_date": fields.Date("Register Date",required=False,search=True),
        "birthday": fields.Date("Birthday",required=False,search=True),
        "phone": fields.Char("Phone",required=False,search=True),
        "mobile": fields.Char("Mobile",required=False,search=True),
        "job": fields.Char("Job/Position"),
        "age": fields.Integer("Age", function="_get_age"),
        'picture': fields.File("Picture"),
        'email': fields.Char("Email"),
        "weight": fields.Float("Weight (cm)"),
        "height": fields.Float("Height (Kg)"),
        "card_type": fields.Selection([("identification","Identification"),("passport","Passport")],"Card Type"),
        'card_no' : fields.Char("Card No."),
        'card_exp' : fields.Date("Card Expiry"),
        "app_no": fields.Char("Application No."),
        "salary": fields.Selection([["20000","5,001-20,000"],["50000","20,001-50,000"],["100000","50,001-100,000"],["100001","100,000+"]], "Salary"),
        "addresses": fields.One2Many("address","related_id","Addresses"),
        "gender": fields.Selection([("male","Male"),("female","Female")],"Gender",required=False),
        "marital_status": fields.Selection([("single","Single"),("marry","Marry"),("divorce","Divorce"),("separated","Saparated"),("widowed","Widowed")],"Marital Status",required=False),
        "nation_id": fields.Many2One("clinic.nation","Nationality"),
        "race_id": fields.Many2One("clinic.race","Race"),
        "grad_id": fields.Many2One("clinic.graduation","Graduation"),
        "smoke": fields.Selection([("never","Never"),("stopped","Stopped"),("smoked","Smoked")],"Smoking"),
        "withdrawal" : fields.Selection([("social_security","Social Security"),("health_insurance","Health Insurance"),("etc","ETC.")],"Right of withdrawal"),
        "first_treatment" : fields.Selection([("hd","HD"),("test","Test")], "First treatment"),
        "first_hemodialysis": fields.Date("First time Hemodialysis",required=False),
        "hemodialysis": fields.Char("First Hemodialysis",required=False),
        "clinic_after": fields.Selection([("small","Small"),("medium","Medium"),("large","Large")],"Clinic Lastime",required=False),
        "clinic_after_name": fields.Char("Clinic after name",required=False),
        "first_permanent_vascular_access": fields.Date("First time Permanent Vascular",required=False),
        "first_tenckhoff_catheters": fields.Date("First time Tenckhoff Catheters",required=False),
        "start_date_clinic": fields.Date("Start Date Clinic",required=False),
        "waiting_transplantation": fields.Selection([("yes","Yes"),("no","No")],"Kidney Transplantation Waiting ?"),
        "who_transplantation": fields.Char("Who is Transplantation?"),
        "reason_of_chronic_renal_failure": fields.Char("Reason chronic renal failure ?"),
        'cause_lines': fields.One2Many("clinic.patient.cause.line","patient_id","Cause Line"),
        'comorbility_lines': fields.One2Many("clinic.patient.comorbidity.line","patient_id","Comorbility"),
        'morbility_lines': fields.One2Many("clinic.patient.morbidity.line","patient_id","Morbility"),
        "comments": fields.One2Many("message","related_id","Comments"),
        "company_id": fields.Many2One("company","Company"),
        "visits": fields.One2Many("clinic.visit","patient_id","Visits"),
        "hd_cases": fields.One2Many("clinic.hd.case","patient_id","HD Cases"),
        "partner_id": fields.Many2One("partner","Contact"),
        "dialyzers": fields.One2Many("clinic.dialyzer","patient_id","Dialyzers"),
        'schedules': fields.Many2Many("clinic.schedule","Schedule"),
    }

    def _get_number(self,context={}):
        while 1:
            seq_id=get_model("sequence").find_sequence(name="Clinic Patient")
            num=get_model("sequence").get_next_number(seq_id,context=context)
            if not num:
                return None
            user_id=get_active_user()
            set_active_user(1)
            res=self.search([["number","=",num]])
            set_active_user(user_id)
            if not res:
                return num
            get_model("sequence").increment_number(seq_id,context=context)
    
    def _get_cause_line(self,context={}):
        cause_ids=get_model("clinic.cause.chronic").search([])
        return cause_ids 

    _defaults={
        "type": "mg",
        "reg_date": lambda *a: time.strftime("%Y-%m-%d"),
        "number": _get_number,
        "company_id": lambda *a: get_active_company(),
        #'cause_lines': _get_cause_line,
    }
    _order="date desc,number desc"

    def void(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.write({"state":"voided"})
    
    def get_partner_id(self,patient_type,data):
        partner_obj=get_model("partner")
        categ_name=PATIENT_TYPE.get(patient_type)
        categ_ids=get_model("partner.categ").search([['name','=',categ_name]])
        if not categ_ids:
            raise Exception("Category: %s not found"%categ_name)
        partner_id=None
        if patient_type in ("mg","sc","nhso"):
            partner_obj=get_model("partner")
            for partner in partner_obj.search_browse([]):
                if partner.categ_id.id in categ_ids:
                    partner_id=partner.id      
                    print("Found partner ", partner.name, " ", partner.id)
                    break
        if not partner_id:
            name=data.get("name","")
            partner_ids=partner_obj.search([['name','=',name]])
            if not partner_ids:
                partner_id=partner_obj.create({
                    'name': name,
                    'last_name': name,
                    'type': 'person'
                })
                print("create new partner ", name)
            else:
                partner_id=partner_ids[0]
        return partner_id

    def onchange_type(self,context={}):
        data=context['data']
        patient_type=data['type']
        partner_id=None
        if patient_type:
            partner_id=self.get_partner_id(patient_type,data)
        data['partner_id']=partner_id
        return data

    def _create(self,vals,**kw):
        obj_id=super(Patient,self).create(vals,**kw)
        obj=self.browse(obj_id)
        partner_id=self.get_partner_id(patient_type=obj.type,data={'name': obj.name})
        obj.write({
            'partner_id': partner_id,
        })
        return obj_id

    def create(self,vals,**kw):
        obj_id=super(Patient,self).create(vals,**kw)
        obj=self.browse(obj_id)
        partner_id=obj.partner_id
        if not partner_id:
            for partner in get_model("partner").search_browse([['name', '=', obj.name]]):
                if partner.name==obj.name:
                    partner_id=partner.id
                    break
            if not partner_id:
                partner_id=get_model("partner").create({
                    'name': obj.name,
                    'last_name': obj.name,
                    'type': 'person',
                })
            obj.write({
                'partner_id': partner_id,
            })
        return obj_id

    def delete(self,ids,context={}):
        partner_ids=[]
        for obj in self.browse(ids):
            partner_ids.append(obj.partner_id.id)
        get_model("partner").delete(partner_ids)
        super().delete(ids)

    def _delete(self,ids,context={}):
        partner_ids=[]
        for obj in self.browse(ids):
            if obj.type not in ("mg","nhso","sc"):
                partner_ids.append(obj.partner_id.id)
        get_model("partner").delete(partner_ids)
        super().delete(ids)
    
    def write(self,ids,vals,**kw):
        for obj in self.browse(ids):
            partner_id=obj.partner_id
            if not partner_id:
                for partner in get_model("partner").search_browse([['name', '=', obj.name]]):
                    if partner.name==obj.name:
                        partner_id=partner.id
                        break
                if not partner_id:
                    partner_id=get_model("partner").create({
                        'name': obj.name,
                        'last_name': obj.name,
                        'type': 'person',
                    })
                vals['partner_id']=partner_id
        super().write(ids,vals,**kw)

Patient.register()
